package webserver

import (

	"github.com/gorilla/handlers"
	"github.com/gorilla/mux"
	queue "git.martyn.berlin/martyn/LEDController/internal/queue"

	"fmt"
	"net/http"
	"os"
	"time"
	"strconv"
)

func HealthHandler(response http.ResponseWriter, request *http.Request) {
	response.Header().Add("Content-type", "text/plain")
	fmt.Fprint(response, "I'm okay jack!")
}

func NotFoundHandler(response http.ResponseWriter, request *http.Request) {
	response.WriteHeader(404)
	fmt.Fprint(response, "I'm lost!")
}

func RootHandler(response http.ResponseWriter, request *http.Request) {
	response.Header().Add("Content-type", "text/plain")
	fmt.Fprint(response, "Not implemented")
}

var globalQueue chan queue.QueueItem;

func PatternHandler(response http.ResponseWriter, request *http.Request) {
	vars := mux.Vars(request)
	response.Header().Add("Content-type", "text/plain")
	var e queue.QueueItem
	e.Effect = vars["pattern"]
	i, _ := strconv.Atoi(vars["duration"])
	e.Duration = uint16(i)
	_, found := vars["speed"]
	if found {
		i, _ := strconv.Atoi(vars["speed"])
		e.Speed = uint16(i)
		fmt.Printf("Speed passed %d\n",e.Speed)
	} else {
		e.Speed = 40
	}
	globalQueue <- e
	fmt.Fprint(response, "OKAY")
}

func HandleHTTP(queueChannel chan queue.QueueItem) {
	globalQueue = queueChannel
	r := mux.NewRouter()
	loggedRouter := handlers.LoggingHandler(os.Stdout, r)
	r.NotFoundHandler = http.HandlerFunc(NotFoundHandler)
	r.HandleFunc("/", RootHandler)
	r.HandleFunc("/healthz", HealthHandler)
	r.HandleFunc("/pattern/{pattern}/{duration}", PatternHandler)
	r.HandleFunc("/pattern/{pattern}/{duration}/{speed}", PatternHandler)
	http.Handle("/", r)
	srv := &http.Server{
		Handler:      loggedRouter,
		Addr:         "0.0.0.0:5353",
		WriteTimeout: 15 * time.Second,
		ReadTimeout:  15 * time.Second,
	}
	fmt.Println("Listening on 0.0.0.0:5353")
	srv.ListenAndServe()
}
